bool LoadTGATexture(int index, char *filename)
{
	FILE *file=fopen(filename, "rb");
	if(!file)
		return false;
	int width=0, height=0;
	int channels;
	unsigned char *data;
	unsigned char id_length;
	int n=0;
	char byte;
	fread(&id_length, 1, 1, file);	// length of ID field
	fread(&byte, 1, 1, file);		// color map (1 or 0)
	fread(&byte, 1, 1, file);		// image type (should be 2(unmapped RGB))
	if(byte!=2) // Not the right type of TGA
		return false;
	for(n=0;n<5;n++)
		fread(&byte, 1, 1, file);	// color map info
	fread(&n, 1, 2, file);			// image origin X
	fread(&n, 1, 2, file);			// image origin Y
	fread(&width, 1, 2, file);		// width
	fread(&height, 1, 2, file);		// height
	fread(&byte, 1, 1, file);		// bits
	channels=4; // force RGBA
	fread(&byte, 1, 1, file);		// image descriptor byte (per-bit info)
	for(n=0;n<id_length;n++)
		fread(&byte, 1, 1, file);	// image description
	data=(unsigned char*)malloc(width*height*channels);
	for(int y=0;y<height;y++)
		for(int x=0;x<width;x++)
		{
			n=(y*width+x)*channels;
			fread(&byte, 1, 1, file);
			data[n+2]=byte;
			fread(&byte, 1, 1, file);
			data[n+1]=byte;
			fread(&byte, 1, 1, file);
			data[n+0]=byte;
			if(data[n+0]==0 && data[n+1]==128 && data[n+2]==128) // Color key
				data[n+3]=0; // transparent
			else
				data[n+3]=255; // opaque
		}
	fclose(file);
	glEnable(GL_TEXTURE_2D);
	textures[index].width=width;
	textures[index].height=height;
	glGenTextures(1, &(textures[index].handle));
	glBindTexture(GL_TEXTURE_2D, textures[index].handle);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	gluBuild2DMipmaps(GL_TEXTURE_2D, channels, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data); // Use copyimage() thingy instead?

	// Build white copy of texture, for blending effects
	for(int y=0;y<height;y++)
		for(int x=0;x<width;x++)
			if(data[(y*width+x)*4+3]==255) // Opaque
			{
				data[(y*width+x)*4+0]=255;
				data[(y*width+x)*4+1]=255;
				data[(y*width+x)*4+2]=255;
			}
	textures2[index].width=width;
	textures2[index].height=height;
	glGenTextures(1, &(textures2[index].handle));
	glBindTexture(GL_TEXTURE_2D, textures2[index].handle);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	gluBuild2DMipmaps(GL_TEXTURE_2D, channels, width, height, GL_RGBA, GL_UNSIGNED_BYTE, data); // Use copyimage() thingy instead?
	free(data);
	return true;
}

void DrawChar(int x, int y, char ch, int size, int color1, int color2)
{
	int index;
	if(ch==' ')
		return;
	if(ch>='A' && ch<='Z')
		index=ch-'A'+12;
	if(ch>='0' && ch<='9')
		index=ch-'0';
	if(ch=='-')
		index=10;
	if(ch=='.')
		index=11;
	glBindTexture(GL_TEXTURE_2D, textures[100+index/16].handle);
	index=index&15;
	float tx=(float)index/16;
	glBegin(GL_QUADS);
	glColor3f(colors[color1].r, colors[color1].g, colors[color1].b);
	glTexCoord2f(tx,1);
	glVertex3i(x, y, 0);
	glColor3f(colors[color2].r, colors[color2].g, colors[color2].b);
	glTexCoord2f(tx,0);
	glVertex3i(x, y+size, 0);
	glTexCoord2f(tx+1.0f/16,0);
	glVertex3i(x+size, y+size, 0);
	glColor3f(colors[color1].r, colors[color1].g, colors[color1].b);
	glTexCoord2f(tx+1.0f/16,1);
	glVertex3i(x+size, y, 0);
	glEnd();
}

void DrawString(int x, int y, char *string, int size, int color1, int color2)
{
	for(int i=0;i<(int)strlen(string);i++)
		DrawChar(x+i*size, y, string[i], size, color1, color2);
}

void DrawSprite(int x, int y, int image, int width, int height, bool flip)
{
	if(width<5) // Small width means zoom factor
	{
		height=textures[image].height*width;
		width=textures[image].width*width;
	}
	if(x>640 || y>480 || x+width<0 || y+height<0) // Off-screen culling
		return;
	float fx=0.0f;
	if(flip)
		fx=1.0f-fx;
	if(image<128)
		glBindTexture(GL_TEXTURE_2D, textures[image].handle);
	else
		glBindTexture(GL_TEXTURE_2D, textures2[image-128].handle);
	glBegin(GL_QUADS);
	glTexCoord2f(fx,1);
	glVertex3i(x, y, 0);
	glTexCoord2f(fx,0);
	glVertex3i(x, y+height, 0);
	glTexCoord2f(1.0f-fx,0);
	glVertex3i(x+width, y+height, 0);
	glTexCoord2f(1.0f-fx,1);
	glVertex3i(x+width, y, 0);
	glEnd();
}

void DrawSpriteScroll(int x, int y, int image, int width, int height, bool flip)
{
	DrawSprite(x-scroll_x, y-scroll_y, image, width, height, flip);
}

